# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'C:/Coding/Python/radie/radie/qt/ui\xyscatter.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_XYScatter(object):
    def setupUi(self, XYScatter):
        XYScatter.setObjectName("XYScatter")
        XYScatter.resize(1000, 561)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(XYScatter)
        self.verticalLayout_2.setContentsMargins(3, 3, 3, 3)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(XYScatter)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.treeView_datasets = DFXYListView(self.layoutWidget)
        self.treeView_datasets.setObjectName("treeView_datasets")
        self.verticalLayout.addWidget(self.treeView_datasets)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.comboBox_lineStyle = QtWidgets.QComboBox(self.layoutWidget)
        self.comboBox_lineStyle.setObjectName("comboBox_lineStyle")
        self.comboBox_lineStyle.addItem("")
        self.comboBox_lineStyle.addItem("")
        self.horizontalLayout.addWidget(self.comboBox_lineStyle)
        self.label_alpha = QtWidgets.QLabel(self.layoutWidget)
        self.label_alpha.setObjectName("label_alpha")
        self.horizontalLayout.addWidget(self.label_alpha)
        self.slider_alpha = QtWidgets.QSlider(self.layoutWidget)
        self.slider_alpha.setMaximum(255)
        self.slider_alpha.setProperty("value", 255)
        self.slider_alpha.setOrientation(QtCore.Qt.Horizontal)
        self.slider_alpha.setObjectName("slider_alpha")
        self.horizontalLayout.addWidget(self.slider_alpha)
        self.label_alphaValue = QtWidgets.QLabel(self.layoutWidget)
        self.label_alphaValue.setObjectName("label_alphaValue")
        self.horizontalLayout.addWidget(self.label_alphaValue)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.groupBox_Legend = QtWidgets.QGroupBox(self.layoutWidget)
        self.groupBox_Legend.setCheckable(True)
        self.groupBox_Legend.setObjectName("groupBox_Legend")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.groupBox_Legend)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.checkBox_legend_dfname = QtWidgets.QCheckBox(self.groupBox_Legend)
        self.checkBox_legend_dfname.setChecked(True)
        self.checkBox_legend_dfname.setObjectName("checkBox_legend_dfname")
        self.horizontalLayout_2.addWidget(self.checkBox_legend_dfname)
        self.checkBox_legend_label = QtWidgets.QCheckBox(self.groupBox_Legend)
        self.checkBox_legend_label.setChecked(False)
        self.checkBox_legend_label.setObjectName("checkBox_legend_label")
        self.horizontalLayout_2.addWidget(self.checkBox_legend_label)
        self.checkBox_legend_ycolumn = QtWidgets.QCheckBox(self.groupBox_Legend)
        self.checkBox_legend_ycolumn.setChecked(True)
        self.checkBox_legend_ycolumn.setObjectName("checkBox_legend_ycolumn")
        self.horizontalLayout_2.addWidget(self.checkBox_legend_ycolumn)
        self.verticalLayout.addWidget(self.groupBox_Legend)
        self.formLayout_plotOptions = QtWidgets.QFormLayout()
        self.formLayout_plotOptions.setObjectName("formLayout_plotOptions")
        self.label_xlabel = QtWidgets.QLabel(self.layoutWidget)
        self.label_xlabel.setObjectName("label_xlabel")
        self.formLayout_plotOptions.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_xlabel)
        self.lineEdit_xlabel = QtWidgets.QLineEdit(self.layoutWidget)
        self.lineEdit_xlabel.setObjectName("lineEdit_xlabel")
        self.formLayout_plotOptions.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.lineEdit_xlabel)
        self.label_ylabel = QtWidgets.QLabel(self.layoutWidget)
        self.label_ylabel.setObjectName("label_ylabel")
        self.formLayout_plotOptions.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_ylabel)
        self.lineEdit_ylabel = QtWidgets.QLineEdit(self.layoutWidget)
        self.lineEdit_ylabel.setObjectName("lineEdit_ylabel")
        self.formLayout_plotOptions.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.lineEdit_ylabel)
        self.verticalLayout.addLayout(self.formLayout_plotOptions)
        self.plotWidget = PlotWidget(self.splitter)
        self.plotWidget.setObjectName("plotWidget")
        self.verticalLayout_2.addWidget(self.splitter)

        self.retranslateUi(XYScatter)
        QtCore.QMetaObject.connectSlotsByName(XYScatter)

    def retranslateUi(self, XYScatter):
        _translate = QtCore.QCoreApplication.translate
        XYScatter.setWindowTitle(_translate("XYScatter", "DataFrames"))
        self.comboBox_lineStyle.setItemText(0, _translate("XYScatter", "line"))
        self.comboBox_lineStyle.setItemText(1, _translate("XYScatter", "points"))
        self.label_alpha.setText(_translate("XYScatter", "Scatter Alpha"))
        self.label_alphaValue.setText(_translate("XYScatter", "255"))
        self.groupBox_Legend.setTitle(_translate("XYScatter", "Legend"))
        self.checkBox_legend_dfname.setText(_translate("XYScatter", "DF Name"))
        self.checkBox_legend_label.setText(_translate("XYScatter", "Item Label"))
        self.checkBox_legend_ycolumn.setText(_translate("XYScatter", "Y-Column"))
        self.label_xlabel.setText(_translate("XYScatter", "X-Label"))
        self.label_ylabel.setText(_translate("XYScatter", "Y-Label"))

from radie.qt.plotlist import DFXYListView
from radie.qt.plotwidget import PlotWidget
